<?php

namespace Drupal\feeds_tamper_ai\Plugin\Tamper;

use Drupal\Core\Form\FormStateInterface;
use Drupal\tamper\Exception\TamperException;
use Drupal\tamper\TamperableItemInterface;
use Drupal\tamper\TamperBase;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatMessage;

/**
 * Plugin implementation of the find_replace plugin.
 *
 * @Tamper(
 *   id = "ai_prompt_tamper",
 *   label = @Translation("AI Prompt"),
 *   description = @Translation("AI Prompt"),
 *   category = "Other"
 * )
 */
class AiPrompt extends TamperBase {

  const SETTING_PROMPTS = 'ai_prompt';
  const SETTING_ROLE = 'ai_role';
  const SETTING_PROVIDER = 'ai_provider';

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = parent::defaultConfiguration();
    $config[self::SETTING_PROMPTS] = '';
    $config[self::SETTING_ROLE] = 'You are an AI assistant.';
    $config[self::SETTING_PROVIDER] = '';
    return $config;
  }


  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $ai_provider_service = \Drupal::service('ai.provider');
    
    if (!$ai_provider_service->hasProvidersForOperationType('chat', TRUE)) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('Sorry, no provider exists for Chat, install one first'),
      ];
    }

    $options = [];
    $providers = $ai_provider_service->getProvidersForOperationType('chat');
    foreach ($providers as $id => $definition) {
      $options[$id] = $definition['label'];
    }
    $form[self::SETTING_PROVIDER] = [
      '#type' => 'select',
      '#title' => $this->t('Choose a provider'),
      '#default_value' => $this->getSetting(self::SETTING_PROVIDER),
      '#options' => $options,
    ];

    $form[self::SETTING_ROLE] = [
      '#type' => 'textfield',
      '#title' => $this->t('What is the system role?'),
      '#description' => $this->t('Give the model instructions about how it should behave and any context it should reference when generating a response. You can describe the assistant’s personality, tell it what it should and shouldn’t answer, and tell it how to format responses.'),
      '#default_value' => $this->getSetting(self::SETTING_ROLE),
    ];

    $form[self::SETTING_PROMPTS] = [
      '#type' => 'textarea',
      '#title' => $this->t('What the assistent should do with data?'),
      '#description' => $this->t('Tell the assistant what should do with the data from source. The assistant can create a short description or translate the content.'),
      '#default_value' => $this->getSetting(self::SETTING_PROMPTS),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->setConfiguration([
      self::SETTING_PROVIDER => $form_state->getValue(self::SETTING_PROVIDER),
      self::SETTING_ROLE => $form_state->getValue(self::SETTING_ROLE),
      self::SETTING_PROMPTS => $form_state->getValue(self::SETTING_PROMPTS),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function tamper($data, TamperableItemInterface $item = NULL) {
    	    // Don't process empty or null values.
    if (is_null($data) || $data === '' || empty($data)) {
      return $data;
    }

    if (!is_string($data) && !is_numeric($data)) {
      throw new TamperException('Input should be a string or numeric.');
    }

    $prompt = $this->getSetting(self::SETTING_PROMPTS);
    $setting_provider = $this->getSetting(self::SETTING_PROVIDER);
    $role = $this->getSetting(self::SETTING_ROLE);


    $messages = new ChatInput([
      new ChatMessage('user', $prompt),
      new ChatMessage('user', $data),
    ]);
    $provider =  \Drupal::service('ai.provider')->createInstance($setting_provider);
    // Set a system message.
    $provider->setChatSystemRole($role);
    /** @var \Drupal\ai\OperationType\Chat\ChatOutput $response */
    $response = $provider->chat($messages, 'gpt-4o', ['my-custom-call']);
    /** @var \Drupal\ai\OperationType\Chat\ChatMessage $return_message */
    $return_message = $response->getNormalized();
    
    return $return_message->getText();
  }

  /**
   * {@inheritdoc}
   */
  public function multiple() {
    return FALSE;
  }




}
